@@ -29,6 +29,7 @@
 #include <bluetooth-applet.h>
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
+#include <bluetooth-utils.h>
 #include <bluetooth-killswitch.h>
 #include <bluetooth-agent.h>
 
@@ -70,8 +71,11 @@ struct _BluetoothApplet
 
 	BluetoothKillswitch* killswitch_manager;
 	BluetoothClient* client;
-	GtkTreeModel* client_model;
-	GtkTreeIter* default_adapter;
+	GtkTreeModel* device_model;
+	gulong signal_row_added;
+	gulong signal_row_changed;
+	gulong signal_row_deleted;
+	DBusGProxy* default_adapter;
 	BluetoothAgent* agent;
 	GHashTable* pending_requests;
 
@@ -107,48 +111,6 @@ enum {
 
 guint signals[SIGNAL_LAST];
 
-typedef struct {
-  GSimpleAsyncResult *result;
-  guint timestamp;
-} MountClosure;
-
-static void
-mount_ready_cb (GObject *object,
-		GAsyncResult *result,
-		gpointer user_data)
-{
-	GError *error;
-	GFile *file = G_FILE (object);
-	char *uri = g_file_get_uri (file);
-	MountClosure *closure = user_data;
-
-	if (g_file_mount_enclosing_volume_finish (file, result, &error) == FALSE) {
-		/* Ignore "already mounted" error */
-		if (error->domain == G_IO_ERROR &&
-		    error->code == G_IO_ERROR_ALREADY_MOUNTED) {
-			g_error_free (error);
-			error = NULL;
-		}
-	}
-
-	if (!error) {
-		gtk_show_uri (NULL, uri, closure->timestamp, &error);
-	}
-
-	if (error) {
-		g_simple_async_result_set_from_error (closure->result, error);
-		g_error_free (error);
-	} else {
-		g_simple_async_result_set_op_res_gboolean (closure->result, TRUE);
-	}
-
-	g_simple_async_result_complete (closure->result);
-
-	g_free (uri);
-	g_object_unref (closure->result);
-	g_free (closure);
-}
-
 /**
  * bluetooth_applet_browse_address_finish:
  *
@@ -163,16 +125,9 @@ bluetooth_applet_browse_address_finish (BluetoothApplet *applet,
 					GAsyncResult *result,
 					GError **error)
 {
-	GSimpleAsyncResult *simple;
-
 	g_return_val_if_fail (BLUETOOTH_IS_APPLET (applet), FALSE);
-	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (applet), bluetooth_applet_browse_address), FALSE);
 
-	simple = G_SIMPLE_ASYNC_RESULT (result);
-	if (g_simple_async_result_propagate_error (simple, error))
-		return FALSE;
-	else
-		return TRUE;
+	return bluetooth_browse_address_finish (G_OBJECT (applet), result, error);
 }
 
 /**
@@ -190,23 +145,13 @@ void bluetooth_applet_browse_address (BluetoothApplet *applet,
 				      GAsyncReadyCallback callback,
 				      gpointer user_data)
 {
-	GFile *file;
-	char *uri;
-	MountClosure *closure;
-
 	g_return_if_fail (BLUETOOTH_IS_APPLET (applet));
-	g_return_if_fail (address != NULL);
 
-	uri = g_strdup_printf ("obex://[%s]/", address);
-	file = g_file_new_for_uri (uri);
-
-	closure = g_new (MountClosure, 1);
-	closure->result = g_simple_async_result_new (G_OBJECT (applet), callback, user_data, bluetooth_applet_browse_address);
-	closure->timestamp = timestamp;
-	g_file_mount_enclosing_volume(file, G_MOUNT_MOUNT_NONE, NULL, NULL, mount_ready_cb, closure);
-
-	g_free (uri);
-	g_object_unref (file);
+	bluetooth_browse_address (applet,
+				  address,
+				  timestamp,
+				  callback,
+				  user_data);
 }
 
 /**
@@ -221,84 +166,22 @@ void bluetooth_applet_send_to_address (BluetoothApplet *applet,
 				       const char *address,
 				       const char *alias)
 {
-	GPtrArray *a;
-	GError *err = NULL;
-	guint i;
-
 	g_return_if_fail (BLUETOOTH_IS_APPLET (applet));
 
-	a = g_ptr_array_new ();
-	g_ptr_array_add (a, "bluetooth-sendto");
-	if (address != NULL) {
-		char *s;
-
-		s = g_strdup_printf ("--device=\"%s\"", address);
-		g_ptr_array_add (a, s);
-	}
-	if (address != NULL && alias != NULL) {
-		char *s;
-
-		s = g_strdup_printf ("--name=\"%s\"", alias);
-		g_ptr_array_add (a, s);
-	}
-	g_ptr_array_add (a, NULL);
-
-	if (g_spawn_async(NULL, (char **) a->pdata, NULL,
-			  G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &err) == FALSE) {
-		g_printerr("Couldn't execute command: %s\n", err->message);
-		g_error_free (err);
-	}
-
-	for (i = 1; a->pdata[i] != NULL; i++)
-		g_free (a->pdata[i]);
-
-	g_ptr_array_free (a, TRUE);
-}
-
-/**
- * bluetooth_applet_agent_reply_passkey:
- *
- * @self: a #BluetoothApplet
- * @request_key: (transfer full): an opaque token given in the pincode-request signal
- * @passkey: (transfer full) (allow-none): the passkey entered by the user, or NULL if the dialog was dismissed
- */
-void
-bluetooth_applet_agent_reply_passkey (BluetoothApplet *self,
-				      char            *request_key,
-				      char            *passkey)
-{
-	DBusGMethodInvocation* context;
-
-	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
-	g_return_if_fail (request_key != NULL);
-
-	context = g_hash_table_lookup (self->pending_requests, request_key);
-
-	if (passkey != NULL) {
-		dbus_g_method_return (context, passkey);
-	} else {
-		GError *error;
-		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
-				     "Pairing request rejected");
-		dbus_g_method_return_error (context, error);
-	}
-
-	g_hash_table_remove (self->pending_requests, request_key);
-	g_free (request_key);
-	g_free (passkey);
+	bluetooth_send_to_address (address, alias);
 }
 
 /**
  * bluetooth_applet_agent_reply_pincode:
  *
  * @self: a #BluetoothApplet
- * @request_key: (transfer full): an opaque token given in the pincode-request signal
- * @pincode: the PIN code entered by the user, or -1 if the dialog was dismissed
+ * @request_key: an opaque token given in the pincode-request signal
+ * @pincode: (allow-none): the PIN code entered by the user, as a string, or NULL if the dialog was dismissed
  */
 void
 bluetooth_applet_agent_reply_pincode (BluetoothApplet *self,
-				      char            *request_key,
-				      int              pincode)
+				      const char      *request_key,
+				      const char      *pincode)
 {
 	DBusGMethodInvocation* context;
 
@@ -307,7 +190,7 @@ bluetooth_applet_agent_reply_pincode (BluetoothApplet *self,
 
 	context = g_hash_table_lookup (self->pending_requests, request_key);
 
-	if (pincode != -1) {
+	if (pincode != NULL) {
 		dbus_g_method_return (context, pincode);
 	} else {
 		GError *error;
@@ -317,19 +200,49 @@ bluetooth_applet_agent_reply_pincode (BluetoothApplet *self,
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
-	g_free (request_key);
+}
+
+/**
+ * bluetooth_applet_agent_reply_passkey:
+ *
+ * @self: a #BluetoothApplet
+ * @request_key: an opaque token given in the pincode-request signal
+ * @passkey: the numeric PIN code entered by the user, or -1 if the dialog was dismissed
+ */
+void
+bluetooth_applet_agent_reply_passkey (BluetoothApplet *self,
+				      const char      *request_key,
+				      int              passkey)
+{
+	DBusGMethodInvocation* context;
+
+	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
+	g_return_if_fail (request_key != NULL);
+
+	context = g_hash_table_lookup (self->pending_requests, request_key);
+
+	if (passkey != -1) {
+		dbus_g_method_return (context, passkey);
+	} else {
+		GError *error;
+		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
+				     "Pairing request rejected");
+		dbus_g_method_return_error (context, error);
+	}
+
+	g_hash_table_remove (self->pending_requests, request_key);
 }
 
 /**
  * bluetooth_applet_agent_reply_confirm:
  *
  * @self: a #BluetoothApplet
- * @request_key: (transfer full): an opaque token given in the pincode-request signal
+ * @request_key: an opaque token given in the pincode-request signal
  * @confirm: TRUE if operation was confirmed, FALSE otherwise
  */
 void
 bluetooth_applet_agent_reply_confirm (BluetoothApplet *self,
-				      char            *request_key,
+				      const char      *request_key,
 				      gboolean         confirm)
 {
 	DBusGMethodInvocation* context;
@@ -349,20 +262,19 @@ bluetooth_applet_agent_reply_confirm (BluetoothApplet *self,
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
-	g_free (request_key);
 }
 
 /**
  * bluetooth_applet_agent_reply_auth:
  *
  * @self: a #BluetoothApplet
- * @request_key: (transfer full): an opaque token given in the pincode-request signal
+ * @request_key: an opaque token given in the pincode-request signal
  * @auth: TRUE if operation was authorized, FALSE otherwise
  * @trusted: TRUE if the operation should be authorized automatically in the future
  */
 void
 bluetooth_applet_agent_reply_auth (BluetoothApplet *self,
-				   char            *request_key,
+				   const char      *request_key,
 				   gboolean         auth,
 				   gboolean         trusted)
 {
@@ -386,7 +298,6 @@ bluetooth_applet_agent_reply_auth (BluetoothApplet *self,
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
-	g_free (request_key);
 }
 
 #ifndef DBUS_TYPE_G_DICTIONARY
@@ -444,9 +355,9 @@ pincode_request (DBusGMethodInvocation *context,
 
 	name = device_get_name (device, &long_name);
 	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), g_object_ref (context));
+	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
 
-	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, TRUE);
+	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, FALSE);
 
 	g_free (name);
 	g_free (long_name);
@@ -466,9 +377,9 @@ passkey_request (DBusGMethodInvocation *context,
 
 	name = device_get_name (device, &long_name);
 	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), g_object_ref (context));
+	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
 
-	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, FALSE);
+	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, TRUE);
 
 	g_free (name);
 	g_free (long_name);
@@ -489,7 +400,7 @@ confirm_request (DBusGMethodInvocation *context,
 
 	name = device_get_name (device, &long_name);
 	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), g_object_ref (context));
+	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
 
 	g_signal_emit (self, signals[SIGNAL_CONFIRM_REQUEST], 0, path, name, long_name, pin);
 
@@ -512,7 +423,7 @@ authorize_request (DBusGMethodInvocation *context,
 
 	name = device_get_name (device, &long_name);
 	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), g_object_ref (context));
+	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
 
 	g_signal_emit (self, signals[SIGNAL_AUTHORIZE_REQUEST], 0, path, name, long_name, uuid);
 
@@ -528,9 +439,10 @@ cancel_request_single (gpointer key, gpointer value, gpointer user_data)
 	DBusGMethodInvocation* request_context = value;
 	GError* result;
 
-	result = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT, "Agent callback cancelled");
-
-	dbus_g_method_return_error (request_context, result);
+	if (value) {
+		result = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT, "Agent callback cancelled");
+		dbus_g_method_return_error (request_context, result);
+	}
 }
 
 static gboolean
@@ -547,95 +459,93 @@ cancel_request(DBusGMethodInvocation *context,
 	return TRUE;
 }
 
-
 static void
-find_default_adapter (BluetoothApplet* self)
+device_added_or_changed (GtkTreeModel *model,
+			 GtkTreePath  *path,
+			 GtkTreeIter  *iter,
+			 gpointer      data)
 {
-	GtkTreeIter iter;
-	gboolean cont;
+	BluetoothApplet *self = BLUETOOTH_APPLET (data);
 
-	if (self->default_adapter) {
-		gtk_tree_iter_free (self->default_adapter);
-		self->default_adapter = NULL;
-	}
-	if (self->agent) {
-		g_object_unref (self->agent);
-		self->agent = NULL;
-	}
-	self->num_adapters_present = self->num_adapters_powered = 0;
-
-	cont = gtk_tree_model_get_iter_first (self->client_model, &iter);
-	while (cont) {
-		gboolean is_default, powered;
-
-		self->num_adapters_present++;
-
-		gtk_tree_model_get (self->client_model, &iter,
-				    BLUETOOTH_COLUMN_DEFAULT, &is_default,
-				    BLUETOOTH_COLUMN_POWERED, &powered,
-				    -1);
-		if (powered)
-			self->num_adapters_powered++;
-		if (is_default && powered)
-			self->default_adapter = gtk_tree_iter_copy (&iter);
-
-		cont = gtk_tree_model_iter_next (self->client_model, &iter);
-	}
-
-	if (self->default_adapter) {
-		DBusGProxy* adapter;
-
-		gtk_tree_model_get (self->client_model, self->default_adapter,
-				    BLUETOOTH_COLUMN_PROXY, &adapter, -1);
-
-		/* This makes sure that the agent is NULL when released */
-		g_object_add_weak_pointer (G_OBJECT (self->agent), (gpointer *) (&self->agent));
-
-		self->agent = bluetooth_agent_new();
-
-		bluetooth_agent_set_pincode_func (self->agent, pincode_request, self);
-		bluetooth_agent_set_passkey_func (self->agent, passkey_request, self);
-		bluetooth_agent_set_confirm_func (self->agent, confirm_request, self);
-		bluetooth_agent_set_authorize_func (self->agent, authorize_request, self);
-		bluetooth_agent_set_cancel_func (self->agent, cancel_request, self);
-
-		bluetooth_agent_register (self->agent, adapter);
-
-		g_object_unref (adapter);
-	}
+	g_signal_emit (self, signals[SIGNAL_DEVICES_CHANGED], 0);
 }
 
 static void
-device_added_or_changed (GtkTreeModel *model,
-                         GtkTreePath  *path,
-                         GtkTreeIter  *iter,
-                         gpointer      data)
+device_removed (GtkTreeModel *model,
+	        GtkTreePath *path,
+	        gpointer user_data)
+{
+	device_added_or_changed (model, path, NULL, user_data);
+}
+
+static void
+default_adapter_changed (GObject    *client,
+			 GParamSpec *spec,
+			 gpointer    data)
 {
 	BluetoothApplet* self = BLUETOOTH_APPLET (data);
 
-	gboolean prev_visibility = bluetooth_applet_get_discoverable (self);
-	gint prev_num_adapters_powered = self->num_adapters_powered;
-	gint prev_num_adapters_present = self->num_adapters_present;
+	if (self->default_adapter)
+		g_object_unref (self->default_adapter);
+	self->default_adapter = bluetooth_client_get_default_adapter (self->client);
 
-	find_default_adapter (self);
+	if (self->device_model) {
+		g_signal_handler_disconnect (self->device_model, self->signal_row_added);
+		g_signal_handler_disconnect (self->device_model, self->signal_row_changed);
+		g_signal_handler_disconnect (self->device_model, self->signal_row_deleted);
+		g_object_unref (self->device_model);
+	}
+	if (self->default_adapter)
+		self->device_model = bluetooth_client_get_device_model (self->client, self->default_adapter);
+	else
+		self->device_model = NULL;
 
-	if (bluetooth_applet_get_discoverable (self) != prev_visibility)
-		g_object_notify (G_OBJECT (self), "discoverable");
-	if (prev_num_adapters_powered != self->num_adapters_powered ||
-	    prev_num_adapters_present != self->num_adapters_present) {
-		g_object_notify (G_OBJECT (self), "killswitch-state");
-		g_object_notify (G_OBJECT (self), "show-full-menu");
+	if (self->device_model) {
+		self->signal_row_added = g_signal_connect (self->device_model, "row-inserted",
+							   G_CALLBACK(device_added_or_changed), self);
+		self->signal_row_deleted = g_signal_connect (self->device_model, "row-deleted",
+							     G_CALLBACK(device_removed), self);
+		self->signal_row_changed = g_signal_connect (self->device_model, "row-changed",
+							     G_CALLBACK (device_added_or_changed), self);
+	}
+
+	if (self->agent)
+		g_object_unref (self->agent);
+
+	if (self->default_adapter) {
+		self->agent = bluetooth_agent_new ();
+		g_object_add_weak_pointer (G_OBJECT (self->agent), (void**) &(self->agent));
+
+		bluetooth_agent_set_pincode_func (self->agent, pincode_request, self);
+		bluetooth_agent_set_passkey_func (self->agent, passkey_request, self);
+		bluetooth_agent_set_authorize_func (self->agent, authorize_request, self);
+		bluetooth_agent_set_confirm_func (self->agent, confirm_request, self);
+		bluetooth_agent_set_cancel_func (self->agent, cancel_request, self);
+
+		bluetooth_agent_register (self->agent, self->default_adapter);
 	}
 
 	g_signal_emit (self, signals[SIGNAL_DEVICES_CHANGED], 0);
 }
 
 static void
-device_removed(GtkTreeModel *model,
-			   GtkTreePath *path,
-			   gpointer user_data)
+default_adapter_powered_changed (GObject    *client,
+				 GParamSpec *spec,
+				 gpointer    data)
 {
-	device_added_or_changed (model, path, NULL, user_data);
+        BluetoothApplet *self = BLUETOOTH_APPLET (data);
+
+	g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_FULL_MENU]);
+}
+
+static void
+default_adapter_discoverable_changed (GObject    *client,
+				 GParamSpec *spec,
+				 gpointer    data)
+{
+        BluetoothApplet *self = BLUETOOTH_APPLET (data);
+
+	g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DISCOVERABLE]);
 }
 
 static gboolean
@@ -782,18 +692,9 @@ bluetooth_applet_disconnect_device (BluetoothApplet* applet,
 gboolean
 bluetooth_applet_get_discoverable (BluetoothApplet* self)
 {
-	gboolean res = FALSE;
-
 	g_return_val_if_fail (BLUETOOTH_IS_APPLET (self), FALSE);
 
-	if (self->default_adapter == NULL)
-		return FALSE;
-
-	gtk_tree_model_get (self->client_model, self->default_adapter,
-			BLUETOOTH_COLUMN_DISCOVERABLE, &res,
-			-1);
-
-	return res;
+       	return bluetooth_client_get_discoverable (self->client);
 }
 
 /**
@@ -861,14 +762,17 @@ bluetooth_applet_set_killswitch_state (BluetoothApplet* self, BluetoothKillswitc
 gboolean
 bluetooth_applet_get_show_full_menu (BluetoothApplet* self)
 {
-
+	gboolean has_adapter, has_powered_adapter;
 	g_return_val_if_fail (BLUETOOTH_IS_APPLET (self), FALSE);
 
-	if (self->num_adapters_present == 0)
+	has_adapter = self->default_adapter != NULL;
+	g_object_get (G_OBJECT (self->client), "default-adapter-powered", &has_powered_adapter, NULL);
+
+	if (!has_adapter)
 		return FALSE;
 
-	return (self->num_adapters_present <= self->num_adapters_powered) &&
-		(bluetooth_applet_get_killswitch_state(self) == BLUETOOTH_KILLSWITCH_STATE_UNBLOCKED);
+	return has_powered_adapter &&
+		bluetooth_applet_get_killswitch_state(self) == BLUETOOTH_KILLSWITCH_STATE_UNBLOCKED;
 }
 
 static BluetoothSimpleDevice *
@@ -876,11 +780,13 @@ bluetooth_applet_create_device_from_iter (GtkTreeModel *model,
 					  GtkTreeIter  *iter,
 					  gboolean      check_proxy)
 {
-	BluetoothSimpleDevice* dev = g_new (BluetoothSimpleDevice, 1);
+	BluetoothSimpleDevice *dev;
 	GHashTable *services;
 	DBusGProxy *proxy;
 	char **uuids;
 
+	dev = g_new0 (BluetoothSimpleDevice, 1);
+
 	gtk_tree_model_get (model, iter,
 			    BLUETOOTH_COLUMN_ADDRESS, &dev->bdaddr,
 			    BLUETOOTH_COLUMN_PROXY, &proxy,
@@ -957,16 +863,16 @@ bluetooth_applet_get_devices (BluetoothApplet* self)
 	if (self->default_adapter == NULL)
 		return NULL;
 
-	cont = gtk_tree_model_iter_children (self->client_model, &iter, self->default_adapter);
+	cont = gtk_tree_model_get_iter_first (self->device_model, &iter);
 	while (cont) {
 		BluetoothSimpleDevice *dev;
 
-		dev = bluetooth_applet_create_device_from_iter (self->client_model, &iter, TRUE);
+		dev = bluetooth_applet_create_device_from_iter (self->device_model, &iter, TRUE);
 
 		if (dev != NULL)
 			result = g_list_prepend (result, dev);
 
-		cont = gtk_tree_model_iter_next(self->client_model, &iter);
+		cont = gtk_tree_model_iter_next (self->device_model, &iter);
 	}
 
 	result = g_list_reverse (result);
@@ -1018,10 +924,8 @@ bluetooth_applet_set_property (GObject      *gobj,
 static void
 bluetooth_applet_init (BluetoothApplet *self)
 {
-	GObject* gobject_client_model = NULL;
-
 	self->client = bluetooth_client_new ();
-	self->client_model = bluetooth_client_get_model (self->client);
+	self->device_model = NULL;
 
 	self->default_adapter = NULL;
 	self->agent = NULL;
@@ -1029,22 +933,18 @@ bluetooth_applet_init (BluetoothApplet *self)
 	self->killswitch_manager = bluetooth_killswitch_new ();
 	g_signal_connect (self->killswitch_manager, "state-changed", G_CALLBACK(killswitch_state_change), self);
 
-	self->pending_requests = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+	self->pending_requests = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	dbus_g_error_domain_register (AGENT_ERROR, "org.bluez.Error", AGENT_ERROR_TYPE);
 
 	/* Make sure all the unblocked adapters are powered,
 	 * so as to avoid seeing unpowered, but unblocked
 	 * devices */
 	set_adapter_powered (self);
-	find_default_adapter (self);
+	default_adapter_changed (NULL, NULL, self);
 
-	gobject_client_model = G_OBJECT (self->client_model);
-	g_signal_connect(gobject_client_model, "row-inserted",
-			 G_CALLBACK(device_added_or_changed), self);
-	g_signal_connect(gobject_client_model, "row-deleted",
-			 G_CALLBACK(device_removed), self);
-	g_signal_connect (gobject_client_model, "row-changed",
-			  G_CALLBACK (device_added_or_changed), self);
+	g_signal_connect (self->client, "notify::default-adapter", G_CALLBACK (default_adapter_changed), self);
+	g_signal_connect (self->client, "notify::default-adapter-powered", G_CALLBACK (default_adapter_powered_changed), self);
+	g_signal_connect (self->client, "notify::default-adapter-discoverable", G_CALLBACK (default_adapter_discoverable_changed), self);
 }
 
 static void
@@ -1063,9 +963,9 @@ bluetooth_applet_dispose (GObject* self)
 		applet->killswitch_manager = NULL;
 	}
 
-	if (applet->client_model) {
-		g_object_unref (applet->client_model);
-		applet->client_model = NULL;
+	if (applet->device_model) {
+		g_object_unref (applet->device_model);
+		applet->device_model = NULL;
 	}
 
 	if (applet->agent) {
@@ -1087,19 +987,20 @@ bluetooth_applet_class_init (BluetoothAppletClass *klass)
 	properties[PROP_KILLSWITCH_STATE] = g_param_spec_int ("killswitch-state",
 							      "Killswitch state",
 							      "State of Bluetooth hardware switches",
-							      KILLSWITCH_STATE_NO_ADAPTER, KILLSWITCH_STATE_HARD_BLOCKED, KILLSWITCH_STATE_NO_ADAPTER, G_PARAM_READABLE | G_PARAM_WRITABLE);
+							      KILLSWITCH_STATE_NO_ADAPTER, KILLSWITCH_STATE_HARD_BLOCKED, KILLSWITCH_STATE_NO_ADAPTER,
+							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class, PROP_KILLSWITCH_STATE, properties[PROP_KILLSWITCH_STATE]);
 
 	properties[PROP_DISCOVERABLE] = g_param_spec_boolean ("discoverable",
 							      "Adapter visibility",
-							      "Wheter the adapter is visible or not",
-							      FALSE, G_PARAM_READABLE | G_PARAM_WRITABLE);
+							      "Whether the adapter is visible or not",
+							      FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class, PROP_DISCOVERABLE, properties[PROP_DISCOVERABLE]);
 
 	properties[PROP_FULL_MENU] = g_param_spec_boolean ("show-full-menu",
 							   "Show the full applet menu",
 							   "Show actions related to the adapter and other miscellanous in the main menu",
-							   TRUE, G_PARAM_READABLE);
+							   TRUE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property (gobject_class, PROP_FULL_MENU, properties[PROP_FULL_MENU]);
 
 	signals[SIGNAL_DEVICES_CHANGED] = g_signal_new ("devices-changed", G_TYPE_FROM_CLASS (gobject_class),
