@@ -89,6 +89,32 @@ event_to_state (guint soft, guint hard)
 		return KILLSWITCH_STATE_UNBLOCKED;
 }
 
+static const char *
+state_to_string (KillswitchState state)
+{
+	switch (state) {
+	case KILLSWITCH_STATE_NO_ADAPTER:
+		return "KILLSWITCH_STATE_NO_ADAPTER";
+	case KILLSWITCH_STATE_SOFT_BLOCKED:
+		return "KILLSWITCH_STATE_SOFT_BLOCKED";
+	case KILLSWITCH_STATE_UNBLOCKED:
+		return "KILLSWITCH_STATE_UNBLOCKED";
+	case KILLSWITCH_STATE_HARD_BLOCKED:
+		return "KILLSWITCH_STATE_HARD_BLOCKED";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+const char *
+bluetooth_killswitch_state_to_string (KillswitchState state)
+{
+	g_return_val_if_fail (state >= KILLSWITCH_STATE_NO_ADAPTER, NULL);
+	g_return_val_if_fail (state <= KILLSWITCH_STATE_HARD_BLOCKED, NULL);
+
+	return state_to_string (state);
+}
+
 static void
 update_killswitch (BluetoothKillswitch *killswitch,
 		   guint index, guint soft, guint hard)
@@ -111,7 +137,8 @@ update_killswitch (BluetoothKillswitch *killswitch,
 	}
 
 	if (changed != FALSE) {
-		g_message ("updating killswitch status %d", index);
+		g_debug ("updating killswitch status %d to %s",
+			 index, state_to_string (bluetooth_killswitch_get_state (killswitch)));
 		g_signal_emit (G_OBJECT (killswitch),
 			       signals[STATE_CHANGED],
 			       0, bluetooth_killswitch_get_state (killswitch));
@@ -161,8 +188,8 @@ bluetooth_killswitch_get_state (BluetoothKillswitch *killswitch)
 	for (l = priv->killswitches ; l ; l = l->next) {
 		BluetoothIndKillswitch *ind = l->data;
 
-		g_message ("killswitch %d is %d",
-			   ind->index, ind->state);
+		g_debug ("killswitch %d is %s",
+			 ind->index, state_to_string (ind->state));
 
 		if (ind->state == KILLSWITCH_STATE_HARD_BLOCKED) {
 			state = KILLSWITCH_STATE_HARD_BLOCKED;
@@ -177,7 +204,7 @@ bluetooth_killswitch_get_state (BluetoothKillswitch *killswitch)
 		state = ind->state;
 	}
 
-	g_message ("killswitches state %d", state);
+	g_debug ("killswitches state %s", state_to_string (state));
 
 	return state;
 }
@@ -203,7 +230,8 @@ remove_killswitch (BluetoothKillswitch *killswitch,
 		BluetoothIndKillswitch *ind = l->data;
 		if (ind->index == index) {
 			priv->killswitches = g_list_remove (priv->killswitches, ind);
-			g_message ("removing killswitch idx %d", index);
+			g_debug ("removing killswitch idx %d", index);
+			g_free (ind);
 			g_signal_emit (G_OBJECT (killswitch),
 				       signals[STATE_CHANGED],
 				       0, bluetooth_killswitch_get_state (killswitch));
@@ -221,13 +249,61 @@ add_killswitch (BluetoothKillswitch *killswitch,
 	BluetoothKillswitchPrivate *priv = BLUETOOTH_KILLSWITCH_GET_PRIVATE (killswitch);
 	BluetoothIndKillswitch *ind;
 
-	g_message ("adding killswitch idx %d state %d", index, state);
+	g_debug ("adding killswitch idx %d state %s", index, state_to_string (state));
 	ind = g_new0 (BluetoothIndKillswitch, 1);
 	ind->index = index;
 	ind->state = state;
 	priv->killswitches = g_list_append (priv->killswitches, ind);
 }
 
+static const char *
+type_to_string (unsigned int type)
+{
+	switch (type) {
+	case RFKILL_TYPE_ALL:
+		return "ALL";
+	case RFKILL_TYPE_WLAN:
+		return "WLAN";
+	case RFKILL_TYPE_BLUETOOTH:
+		return "BLUETOOTH";
+	case RFKILL_TYPE_UWB:
+		return "UWB";
+	case RFKILL_TYPE_WIMAX:
+		return "WIMAX";
+	case RFKILL_TYPE_WWAN:
+		return "WWAN";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static const char *
+op_to_string (unsigned int op)
+{
+	switch (op) {
+	case RFKILL_OP_ADD:
+		return "ADD";
+	case RFKILL_OP_DEL:
+		return "DEL";
+	case RFKILL_OP_CHANGE:
+		return "CHANGE";
+	case RFKILL_OP_CHANGE_ALL:
+		return "CHANGE_ALL";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static void
+print_event (struct rfkill_event *event)
+{
+	g_debug ("RFKILL event: idx %u type %u (%s) op %u (%s) soft %u hard %u",
+		 event->idx,
+		 event->type, type_to_string (event->type),
+		 event->op, op_to_string (event->op),
+		 event->soft, event->hard);
+}
+
 static gboolean
 event_cb (GIOChannel *source,
 	  GIOCondition condition,
@@ -236,20 +312,23 @@ event_cb (GIOChannel *source,
 	if (condition & G_IO_IN) {
 		GIOStatus status;
 		struct rfkill_event event;
+		gsize read;
+		gboolean changed = FALSE;
 
 		status = g_io_channel_read_chars (source,
 						  (char *) &event,
 						  sizeof(event),
-						  NULL,
+						  &read,
 						  NULL);
-		if (status == G_IO_STATUS_NORMAL) {
-			g_message ("RFKILL event: idx %u type %u op %u soft %u hard %u\n",
-				   event.idx, event.type, event.op,
-				   event.soft, event.hard);
 
+		while (status == G_IO_STATUS_NORMAL && read == sizeof(event)) {
 			if (event.type != RFKILL_TYPE_BLUETOOTH &&
 			    event.type != RFKILL_TYPE_ALL)
-				return TRUE;
+				goto carry_on;
+
+			print_event (&event);
+
+			changed = TRUE;
 
 			if (event.op == RFKILL_OP_CHANGE) {
 				update_killswitch (killswitch, event.idx, event.soft, event.hard);
@@ -261,12 +340,20 @@ event_cb (GIOChannel *source,
 				add_killswitch (killswitch, event.idx, state);
 			}
 
+carry_on:
+			status = g_io_channel_read_chars (source,
+							  (char *) &event,
+							  sizeof(event),
+							  &read,
+							  NULL);
+		}
+
+		if (changed)
 			g_signal_emit (G_OBJECT (killswitch),
 				       signals[STATE_CHANGED],
 				       0, bluetooth_killswitch_get_state (killswitch));
-		}
 	} else {
-		g_message ("something else happened");
+		g_debug ("something else happened");
 		return FALSE;
 	}
 
@@ -290,7 +377,7 @@ bluetooth_killswitch_init (BluetoothKillswitch *killswitch)
 	}
 
 	if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
-		g_message ("Can't set RFKILL control device to non-blocking");
+		g_debug ("Can't set RFKILL control device to non-blocking");
 		close(fd);
 		return;
 	}
@@ -303,12 +390,12 @@ bluetooth_killswitch_init (BluetoothKillswitch *killswitch)
 		if (len < 0) {
 			if (errno == EAGAIN)
 				break;
-			g_message ("Reading of RFKILL events failed");
+			g_debug ("Reading of RFKILL events failed");
 			break;
 		}
 
 		if (len != RFKILL_EVENT_SIZE_V1) {
-			g_warning("Wrong size of RFKILL event\n");
+			g_warning ("Wrong size of RFKILL event\n");
 			continue;
 		}
 
