@@ -60,7 +60,7 @@ set_last_used_device (void)
 	bdaddr = g_settings_get_string (settings, PREF_LAST_USED);
 	g_object_unref (settings);
 
-	if (bdaddr != NULL && *bdaddr != '\0') {
+	if (*bdaddr != '\0') {
 		g_object_set (G_OBJECT (combo), "device", bdaddr, NULL);
 	} else {
 		g_object_set (G_OBJECT (combo), "device", BLUETOOTH_CHOOSER_COMBO_FIRST_DEVICE, NULL);
@@ -165,7 +165,7 @@ validate_destination (NstPlugin *plugin,
 	g_return_val_if_fail (error != NULL, FALSE);
 
 	if (get_select_device (NULL, &bdaddr) == FALSE) {
-		*error = g_strdup (_("Programming error, could not find the device in the list"));
+		*error = g_strdup (_("Programming error: could not find the device in the list"));
 		return FALSE;
 	}
 
@@ -240,7 +240,7 @@ bail:
 	g_object_unref (device);
 
 	if (found == FALSE)
-		*error = g_strdup_printf (_("Obex Push file transfer unsupported"));
+		*error = g_strdup_printf (_("OBEX Push file transfer unsupported"));
 
 	return found;
 #endif
